//similar to f0neo_firmware but with more osc commands, gamma and a fade switch
//client version (connects to another access point)

#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <OSCMessage.h>
#include <OSCData.h>
#include <Adafruit_NeoPixel.h>

#define PORT 19999
#define NUMNEO 5  //EDIT number of neo pixels in use
#define PINNEO 2
#define PINSWI 3  //rx used as switch. 1 = fade in to last set rgbw value, 0 = fade out to black
#define UPDATERATE 16

const char *ssid = "f0neo"; //EDIT your accessPoint network name
const char *password = "mypass123";  //EDIT your password
float fadeSpeed = 0.007; //EDIT default fade in/out speed
uint8_t defaultColor[] = {255, 255, 255, 255};

WiFiUDP Udp;
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMNEO, PINNEO, NEO_RGBW + NEO_KHZ800);
extern const uint8_t gamma8[];
uint8_t lastOsc[NUMNEO * 4];
uint8_t lastOut[NUMNEO * 4];
uint8_t lastSwi;
uint8_t ledState;
float fade;
unsigned long nextTime;

void setup() {
  delay(10);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(100);
  }
  Udp.begin(PORT);
  Serial.begin(115200, SERIAL_8N1, SERIAL_TX_ONLY);
  pinMode(PINSWI, INPUT_PULLUP);
  lastSwi = digitalRead(PINSWI);
  ledState = 0;
  fade = 0.0;
  pixels.begin();
  for (int i = 0; i < NUMNEO; i++) {
    lastOsc[i * 4] = defaultColor[0];
    lastOsc[i * 4 + 1] = defaultColor[1];
    lastOsc[i * 4 + 2] = defaultColor[2];
    lastOsc[i * 4 + 3] = defaultColor[3];
    pixels.setPixelColor(i, defaultColor[0], defaultColor[1], defaultColor[2], defaultColor[3]);
  }
  pixels.show();
}

void rgbw(OSCMessage &msg) {
  int index = msg.getInt(0) * 4;
  lastOsc[index] = msg.getInt(2);
  lastOsc[index + 1] = msg.getInt(1);
  lastOsc[index + 2] = msg.getInt(3);
  lastOsc[index + 3] = msg.getInt(4);
}
void onoff(OSCMessage &msg) {
  ledState = msg.getInt(0);
}
void speed(OSCMessage &msg) {
  fadeSpeed = msg.getFloat(0);
}

void loop() {

  //--osc input
  OSCMessage oscMsg;
  int packetSize = Udp.parsePacket();
  if (packetSize) {
    while (packetSize--) {
      oscMsg.fill(Udp.read());
    }
    if (!oscMsg.hasError()) {
      oscMsg.dispatch("/rgbw", rgbw);
      oscMsg.dispatch("/onoff", onoff);
      oscMsg.dispatch("/speed", speed);
    }
  }

  if (millis() >= nextTime) {
    nextTime = millis() + UPDATERATE;

    //--fade switch
    if (digitalRead(PINSWI) != lastSwi) {
      lastSwi = digitalRead(PINSWI);
      ledState = lastSwi;
    }

    //--fade in/out
    if (fade < ledState) {
      fade = fade + fadeSpeed;
      if (fade > 1.0) {
        fade = 1.0;
      }
    } else if (fade > ledState) {
      fade = fade - fadeSpeed;
      if (fade < 0.0) {
        fade = 0.0;
      }
    }

    //--update pixels
    bool changed = false;
    for (int i = 0; i < NUMNEO * 4; i++) {
      uint8_t val = lastOsc[i] * fade;
      //val = pgm_read_byte(&gamma8[val]);  //optional gamma correction //EDIT uncomment to activate
      if (lastOut[i] != val) {
        lastOut[i] = val;
        changed = true;
      }
    }
    if (changed) {
      for (int i = 0; i < NUMNEO; i++) {
        pixels.setPixelColor(i, lastOut[i * 4], lastOut[i * 4 + 1], lastOut[i * 4 + 2], lastOut[i * 4 + 3]);
      }
      pixels.show();
    }
  }
}
